@@ -4,14 +4,20 @@
 # Copyright (c) 2012 Mike Bryant <leachim@leachim.info>
 # Copyright (c) 2014 Brett Cannon <brett@python.org>
 # Copyright (c) 2014 Arun Persaud <arun@nubati.net>
-# Copyright (c) 2015-2018 Claudiu Popa <pcmanticore@gmail.com>
+# Copyright (c) 2015-2020 Claudiu Popa <pcmanticore@gmail.com>
 # Copyright (c) 2015 Ionel Cristian Maries <contact@ionelmc.ro>
+# Copyright (c) 2016, 2019-2020 Ashley Whetter <ashley@awhetter.co.uk>
 # Copyright (c) 2016 Chris Murray <chris@chrismurray.scot>
-# Copyright (c) 2016 Ashley Whetter <ashley@awhetter.co.uk>
 # Copyright (c) 2017 guillaume2 <guillaume.peillex@gmail.col>
 # Copyright (c) 2017 Łukasz Rogalski <rogalski.91@gmail.com>
+# Copyright (c) 2018 Alan Chan <achan961117@gmail.com>
+# Copyright (c) 2018 Yury Gribov <tetra2005@gmail.com>
 # Copyright (c) 2018 Mike Frysinger <vapier@gmail.com>
 # Copyright (c) 2018 Mariatta Wijaya <mariatta@python.org>
+# Copyright (c) 2019 Djailla <bastien.vallet@gmail.com>
+# Copyright (c) 2019 Pierre Sassoulas <pierre.sassoulas@gmail.com>
+# Copyright (c) 2019 Svet <svet@hyperscience.com>
+# Copyright (c) 2020 Anthony Sottile <asottile@umich.edu>
 # Licensed under the GPL: https://www.gnu.org/licenses/old-licenses/gpl-2.0.html
 # For details: https://github.com/PyCQA/pylint/blob/master/COPYING
 
@@ -21,45 +27,49 @@ import string
 
 import astroid
 
-from pylint import checkers
-from pylint import interfaces
+from pylint import checkers, interfaces
 from pylint.checkers import utils
 from pylint.checkers.utils import check_messages
 
-
 MSGS = {
     "W1201": (
-        "Specify string format arguments as logging function parameters",
+        "Use %s formatting in logging functions",
         "logging-not-lazy",
         "Used when a logging statement has a call form of "
         '"logging.<logging method>(format_string % (format_args...))". '
-        "Such calls should leave string interpolation to the logging "
-        "method itself and be written "
-        '"logging.<logging method>(format_string, format_args...)" '
-        "so that the program may avoid incurring the cost of the "
-        "interpolation in those cases in which no message will be "
-        "logged. For more, see "
-        "http://www.python.org/dev/peps/pep-0282/.",
+        "Use another type of string formatting instead. "
+        "You can use % formatting but leave interpolation to "
+        "the logging function by passing the parameters as arguments. "
+        "If logging-fstring-interpolation is disabled then "
+        "you can use fstring formatting. "
+        "If logging-format-interpolation is disabled then "
+        "you can use str.format.",
     ),
     "W1202": (
-        "Use % formatting in logging functions and pass the % "
-        "parameters as arguments",
+        "Use %s formatting in logging functions",
         "logging-format-interpolation",
         "Used when a logging statement has a call form of "
-        '"logging.<logging method>(format_string.format(format_args...))"'
-        ". Such calls should use % formatting instead, but leave "
-        "interpolation to the logging function by passing the parameters "
-        "as arguments.",
+        '"logging.<logging method>(format_string.format(format_args...))". '
+        "Use another type of string formatting instead. "
+        "You can use % formatting but leave interpolation to "
+        "the logging function by passing the parameters as arguments. "
+        "If logging-fstring-interpolation is disabled then "
+        "you can use fstring formatting. "
+        "If logging-not-lazy is disabled then "
+        "you can use % formatting as normal.",
     ),
     "W1203": (
-        "Use % formatting in logging functions and pass the % "
-        "parameters as arguments",
+        "Use %s formatting in logging functions",
         "logging-fstring-interpolation",
         "Used when a logging statement has a call form of "
-        '"logging.method(f"..."))"'
-        ". Such calls should use % formatting instead, but leave "
-        "interpolation to the logging function by passing the parameters "
-        "as arguments.",
+        '"logging.<logging method>(f"...")".'
+        "Use another type of string formatting instead. "
+        "You can use % formatting but leave interpolation to "
+        "the logging function by passing the parameters as arguments. "
+        "If logging-format-interpolation is disabled then "
+        "you can use str.format. "
+        "If logging-not-lazy is disabled then "
+        "you can use % formatting as normal.",
     ),
     "E1200": (
         "Unsupported logging format character %r (%#02x) at index %d",
@@ -143,8 +153,8 @@ class LoggingChecker(checkers.BaseChecker):
                 "type": "choice",
                 "metavar": "<old (%) or new ({)>",
                 "choices": ["old", "new"],
-                "help": "Format style used to check logging format string. "
-                "`old` means using % formatting, while `new` is for `{}` formatting.",
+                "help": "The type of string formatting that logging methods do. "
+                "`old` means using % formatting, `new` is for `{}` formatting.",
             },
         ),
     )
@@ -158,6 +168,7 @@ class LoggingChecker(checkers.BaseChecker):
         logging_mods = self.config.logging_modules
 
         self._format_style = self.config.logging_format_style
+
         self._logging_modules = set(logging_mods)
         self._from_imports = {}
         for logging_mod in logging_mods:
@@ -245,15 +256,36 @@ class LoggingChecker(checkers.BaseChecker):
                 )
                 emit = total_number_of_strings > 0
             if emit:
-                self.add_message("logging-not-lazy", node=node)
+                self.add_message(
+                    "logging-not-lazy", node=node, args=(self._helper_string(node),),
+                )
         elif isinstance(node.args[format_pos], astroid.Call):
             self._check_call_func(node.args[format_pos])
         elif isinstance(node.args[format_pos], astroid.Const):
             self._check_format_string(node, format_pos)
-        elif isinstance(
-            node.args[format_pos], (astroid.FormattedValue, astroid.JoinedStr)
+        elif isinstance(node.args[format_pos], astroid.JoinedStr):
+            self.add_message(
+                "logging-fstring-interpolation",
+                node=node,
+                args=(self._helper_string(node),),
+            )
+
+    def _helper_string(self, node):
+        """Create a string that lists the valid types of formatting for this node."""
+        valid_types = ["lazy %"]
+
+        if not self.linter.is_message_enabled(
+            "logging-fstring-formatting", node.fromlineno
         ):
-            self.add_message("logging-fstring-interpolation", node=node)
+            valid_types.append("fstring")
+        if not self.linter.is_message_enabled(
+            "logging-format-interpolation", node.fromlineno
+        ):
+            valid_types.append(".format()")
+        if not self.linter.is_message_enabled("logging-not-lazy", node.fromlineno):
+            valid_types.append("%")
+
+        return " or ".join(valid_types)
 
     @staticmethod
     def _is_operand_literal_str(operand):
@@ -275,7 +307,11 @@ class LoggingChecker(checkers.BaseChecker):
         if is_method_call(func, types, methods) and not is_complex_format_str(
             func.bound
         ):
-            self.add_message("logging-format-interpolation", node=node)
+            self.add_message(
+                "logging-format-interpolation",
+                node=node,
+                args=(self._helper_string(node),),
+            )
 
     def _check_format_string(self, node, format_arg):
         """Checks that format string tokens match the supplied arguments.
@@ -289,12 +325,12 @@ class LoggingChecker(checkers.BaseChecker):
             # If no args were supplied the string is not interpolated and can contain
             # formatting characters - it's used verbatim. Don't check any further.
             return
+
         format_string = node.args[format_arg].value
-        if not isinstance(format_string, str):
-            # If the log format is constant non-string (e.g. logging.debug(5)),
-            # ensure there are no arguments.
-            required_num_args = 0
-        else:
+        required_num_args = 0
+        if isinstance(format_string, bytes):
+            format_string = format_string.decode()
+        if isinstance(format_string, str):
             try:
                 if self._format_style == "old":
                     keyword_args, required_num_args, _, _ = utils.parse_format_string(
@@ -305,12 +341,14 @@ class LoggingChecker(checkers.BaseChecker):
                         # special keywords - out of scope.
                         return
                 elif self._format_style == "new":
-                    keyword_arguments, implicit_pos_args, explicit_pos_args = utils.parse_format_method_string(
-                        format_string
-                    )
+                    (
+                        keyword_arguments,
+                        implicit_pos_args,
+                        explicit_pos_args,
+                    ) = utils.parse_format_method_string(format_string)
 
                     keyword_args_cnt = len(
-                        set(k for k, l in keyword_arguments if not isinstance(k, int))
+                        {k for k, l in keyword_arguments if not isinstance(k, int)}
                     )
                     required_num_args = (
                         keyword_args_cnt + implicit_pos_args + explicit_pos_args
@@ -341,7 +379,9 @@ def is_complex_format_str(node):
         bool: True if inferred string uses complex formatting, False otherwise
     """
     inferred = utils.safe_infer(node)
-    if inferred is None or not isinstance(inferred.value, str):
+    if inferred is None or not (
+        isinstance(inferred, astroid.Const) and isinstance(inferred.value, str)
+    ):
         return True
     try:
         parsed = list(string.Formatter().parse(inferred.value))
